/* Change framebuffer background image.
   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
   (c) Copyright 2000-2004  Convergence (integrated media) GmbH

   All rights reserved.

   Written by Denis Oliver Kropp <dok@directfb.org>,
              Andreas Hundt <andi@fischlustig.de>,
              Sven Neumann <neo@directfb.org>,
              Ville Syrjälä <syrjala@sci.fi> and
              Claudio Ciccani <klan@users.sf.net>.

   This file is subject to the terms and conditions of the MIT License:

   Permission is hereby granted, free of charge, to any person
   obtaining a copy of this software and associated documentation
   files (the "Software"), to deal in the Software without restriction,
   including without limitation the rights to use, copy, modify, merge,
   publish, distribute, sublicense, and/or sell copies of the Software,
   and to permit persons to whom the Software is furnished to do so,
   subject to the following conditions:

   The above copyright notice and this permission notice shall be
   included in all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

/* Text support and improved option parsing by Guilherme F. Lima
   <guilhermeflima@gmail.com>.  */

#include <stdint.h>
#include <directfb.h>

#include "system.h"
#include "error.h"

/* The official name of this program.  */
#define PROGRAM_NAME "tm-dfbg"
#define VERSION "1.1"
#define BUGS_TO "guilhermeflima@gmail.com"

/* Constants.  */
#define DEFAULT_FONT "/usr/local/etc/ginga/files/font/vera.ttf"

/* String containing name the program is called with.  To be
   initialized by main(), based on argv[0].  */
const char *program_name;

/* Background image file name.  */
static char *image = NULL;

/* Text parameters.  */
struct textinfo
{
     char *string;      /* String to be displayed.  */
     char *font;        /* True-type font filename.  */
     int  size;         /* Text size in ppt.  */
     int  x;            /* x coordinate.  */
     int  y;            /* y coordinate.  */
};
static struct textinfo *text = NULL;

/* Long option support.  */
static struct option const long_options[] =
{
     {"ttf-font", required_argument, NULL, 'f'},
     {"position", required_argument, NULL, 'p'},
     {"size", required_argument, NULL, 's'},
     {"text", required_argument, NULL, 'x'},
     {GETOPT_HELP_OPTION_DECL},
     {GETOPT_VERSION_OPTION_DECL},
     {NULL, 0, NULL, 0}
};

static void
usage (int status)
{
     if (status != EXIT_SUCCESS)
          fprintf (stderr, "Try `%s --help' for more information.\n",
                  program_name);
     else
     {
          printf ("\
Usage: %s [OPTION]... [FILE]\n\
Change framebuffer background image.\n\
\n\
      --help     display this help and exit\n\
      --version  output version information and exit\n\
\n\
  -f, --ttf-font=FILE  use true-type font FILE\n\
  -p, --position=X,Y   X,Y coordinate\n\
  -s, --size=INT       use INT ppt fontsize\n\
  -x, --text=STRING    display text STRING\n\
\n\
Report bugs to <%s>\n", program_name, BUGS_TO);
     }
     exit (status);
}

static void
parse_args (int argc, char **argv)
{
     for (;;)
     {
          int oi = -1;
          int c = getopt_long (argc, argv, "f:p:s:x:", long_options, &oi);

          if (c == -1)
               break;

          switch (c)
          {
          case 'f': /* --ttf-font */
               text->font = optarg;
               break;

          case 'p': /* --position */
               sscanf (optarg, "%d,%d", &text->x, &text->y);
               break;

          case 's': /* --size */
               if ((text->size = atoi (optarg)) <= 0)
               {
                    error (0, 0, "invalid font size: %d", text->size);
                    usage (EXIT_FAILURE);
               }
               break;

          case 'x': /* --text */
               text->string = optarg;
               break;

          case_GETOPT_HELP_CHAR;

          case_GETOPT_VERSION_CHAR(PROGRAM_NAME, VERSION);

          default:
               usage (EXIT_FAILURE);
          }
     }

     if (optind >= argc)
     {
          error (0, 0, "missing image file name");
          usage (EXIT_FAILURE);
     }
     image = argv[optind];

#ifdef DEBUG
     printf ("tstring=%s, tfont=%s, tsize=%d, x=%d, y=%d, image=%s\n",
             text->string, text->font, text->size, text->x, text->y, image);
#endif
}

static int
set_background_image (char *image,
                      struct textinfo *text,
                      IDirectFB *dfb,
                      IDirectFBDisplayLayer *layer)
{
     IDirectFBImageProvider *ip = NULL;
     IDirectFBSurface *surf = NULL;
     DFBSurfaceDescription desc;
     DFBDisplayLayerConfig config;
     int status = EXIT_FAILURE;
     bool using_default_font = false;

     if (dfb->CreateImageProvider (dfb, image, &ip))
     {
          error (0, 0, "IDirectFB::CreateImageProvider() failed.");
          goto exit;
     }

     if (ip->GetSurfaceDescription (ip, &desc))
     {
          error (0, 0, "ImageProvider::GetSurfaceDescription () failed");
          goto exit;
     }
     desc.flags |= DSDESC_CAPS;
     desc.caps = DSCAPS_SHARED;

     if (layer->GetConfiguration (layer, &config))
     {
          error (0, 0, "DisplayLayer::GetConfiguration() failed");
          goto exit;
     }
     desc.width = config.width;
     desc.height = config.height;

     if (dfb->CreateSurface (dfb, &desc, &surf))
     {
          error (0, 0, "IDirectFB::CreateSurface() failed");
          goto exit;
     }

     if (ip->RenderTo (ip, surf, NULL))
     {
          error (0, 0, "ImageProvider::RenderTo() failed");
          goto exit;
     }

     if (layer->SetBackgroundImage (layer, surf))
     {
          error (0, 0, "DisplayLayer::SetBackgroundImage() failed");
          goto exit;
     }

     /* Display text.  */
     if (text->string)
     {
          IDirectFBFont *font;
          DFBFontDescription fdesc;

          surf->SetColor (surf, 0, 0, 0, 0xff);
          fdesc.flags = DFDESC_HEIGHT | DFDESC_ATTRIBUTES;
          fdesc.height = text->size;
          fdesc.attributes = 0;

          /* Use default font.  */
          if (text->font == NULL)
          {
               if (access (DEFAULT_FONT, R_OK) == -1)
                    error (1, errno, "unable to load default font file %s",
                           DEFAULT_FONT);

               text->font = malloc (strlen (DEFAULT_FONT) + 1);
               if (text->font == NULL)
                    error(1, errno, "%s: malloc failed", __FUNCTION__);
               strcpy (text->font, DEFAULT_FONT);
               using_default_font = true;
          }

          dfb->CreateFont (dfb, text->font, &fdesc, &font);
          surf->SetFont (surf, font);
          surf->DrawString (surf,
                            text->string, -1, text->x, text->y, DSTF_LEFT);
     }

     if (layer->SetBackgroundMode (layer, DLBM_IMAGE))
     {
          error(0, 0, "DisplayLayer::SetBackgroundMode() failed");
          goto exit;
     }
     status = EXIT_SUCCESS;
exit:
     if (using_default_font)
          free (text->font);
     surf ? surf->Release (surf) : 0;
     ip ? ip->Release (ip) : 0;
     return status;
}

int
main (int argc, char **argv)
{
     IDirectFB *dfb = NULL;
     IDirectFBDisplayLayer *layer = NULL;
     int status;

     program_name = argv[0];

     if (DirectFBInit (&argc, &argv))
          error (1, 0, "DirecFBInit() failed");

     /* Initialize global text structure.  */
     text = malloc (sizeof (struct textinfo));
     if (text == NULL)
          error (1, errno, "%s: malloc failed", __FUNCTION__);
     text->string = NULL;
     text->font   = NULL;
     text->size   = 12;
     text->x      = 0;
     text->y      = 0;

     parse_args (argc, argv);

     DirectFBSetOption ("bg-none", NULL);
     DirectFBSetOption ("no-cursor", NULL);

     if (DirectFBCreate (&dfb))
          error (1, 0, "DirectFBCreate() failed");

     if (dfb->GetDisplayLayer (dfb, DLID_PRIMARY, &layer))
     {
          dfb->Release (dfb);
          error (1, 0, "IDirectFB::GetDisplayLayer() failed");
     }

     if (layer->SetCooperativeLevel (layer, DLSCL_ADMINISTRATIVE))
     {
          layer->Release (layer);
          dfb->Release (dfb);
          error (1, 0, "DisplayLayer::SetCooperativeLevel() failed");
     }

     status = set_background_image (image, text, dfb, layer);

     free (text);
     layer->Release (layer);
     dfb->Release (dfb);
     exit (status);
}
